@@ -1,29 +1,40 @@
+from typing import Union
+
 import numpy as np
 
 from pystencils.astnodes import Block, KernelFunction, LoopOverCoordinate, SympyAssignment
-from pystencils.data_types import StructType, TypedSymbol
+from pystencils.config import CreateKernelConfig
+from pystencils.typing import StructType, TypedSymbol
+from pystencils.typing.transformations import add_types
 from pystencils.field import Field, FieldType
+from pystencils.enums import Target, Backend
 from pystencils.gpucuda.cudajit import make_python_function
-from pystencils.gpucuda.indexing import BlockIndexing
+from pystencils.node_collection import NodeCollection
+from pystencils.gpucuda.indexing import indexing_creator_from_params
+from pystencils.simp.assignment_collection import AssignmentCollection
 from pystencils.transformations import (
-    add_types, get_base_buffer_index, get_common_shape, implement_interpolations,
-    parse_base_pointer_info, resolve_buffer_accesses, resolve_field_accesses, unify_shape_symbols)
+    get_base_buffer_index, get_common_field, parse_base_pointer_info,
+    resolve_buffer_accesses, resolve_field_accesses, unify_shape_symbols)
 
 
-def create_cuda_kernel(assignments,
-                       function_name="kernel",
-                       type_info=None,
-                       indexing_creator=BlockIndexing,
-                       iteration_slice=None,
-                       ghost_layers=None,
-                       skip_independence_check=False,
-                       use_textures_for_interpolation=True):
-    assert assignments, "Assignments must not be empty!"
-    fields_read, fields_written, assignments = add_types(assignments, type_info, not skip_independence_check)
+def create_cuda_kernel(assignments: Union[AssignmentCollection, NodeCollection],
+                       config: CreateKernelConfig):
+
+    function_name = config.function_name
+    indexing_creator = indexing_creator_from_params(config.gpu_indexing, config.gpu_indexing_params)
+    iteration_slice = config.iteration_slice
+    ghost_layers = config.ghost_layers
+
+    fields_written = assignments.bound_fields
+    fields_read = assignments.rhs_fields
+    assignments = assignments.all_assignments
+
+    assignments = add_types(assignments, config)
+
     all_fields = fields_read.union(fields_written)
     read_only_fields = set([f.name for f in fields_read - fields_written])
 
-    buffers = set([f for f in all_fields if FieldType.is_buffer(f) or FieldType.is_custom(f)])
+    buffers = set([f for f in all_fields if FieldType.is_buffer(f)])
     fields_without_buffers = all_fields - buffers
 
     field_accesses = set()
@@ -33,7 +44,9 @@ def create_cuda_kernel(assignments,
         field_accesses = {e for e in field_accesses if not e.is_absolute_access}
         num_buffer_accesses += sum(1 for access in eq.atoms(Field.Access) if FieldType.is_buffer(access.field))
 
-    common_shape = get_common_shape(fields_without_buffers)
+    # common shape and field to from the iteration space
+    common_field = get_common_field(fields_without_buffers)
+    common_shape = common_field.spatial_shape
 
     if iteration_slice is None:
         # determine iteration slice from ghost layers
@@ -51,7 +64,7 @@ def create_cuda_kernel(assignments,
                 iteration_slice.append(slice(ghost_layers[i][0],
                                              -ghost_layers[i][1] if ghost_layers[i][1] > 0 else None))
 
-    indexing = indexing_creator(field=list(fields_without_buffers)[0], iteration_slice=iteration_slice)
+    indexing = indexing_creator(field=common_field, iteration_slice=iteration_slice)
     coord_mapping = indexing.coordinates
 
     cell_idx_assignments = [SympyAssignment(LoopOverCoordinate.get_loop_counter_symbol(i), value)
@@ -65,16 +78,14 @@ def create_cuda_kernel(assignments,
     unify_shape_symbols(block, common_shape=common_shape, fields=fields_without_buffers)
 
     ast = KernelFunction(block,
-                         'gpu',
-                         'gpucuda',
+                         Target.GPU,
+                         Backend.CUDA,
                          make_python_function,
                          ghost_layers,
                          function_name,
                          assignments=assignments)
     ast.global_variables.update(indexing.index_variables)
 
-    implement_interpolations(ast, implement_by_texture_accesses=use_textures_for_interpolation)
-
     base_pointer_spec = [['spatialInner0']]
     base_pointer_info = {f.name: parse_base_pointer_info(base_pointer_spec, [2, 1, 0],
                                                          f.spatial_dimensions, f.index_dimensions)
@@ -82,10 +93,9 @@ def create_cuda_kernel(assignments,
 
     coord_mapping = {f.name: cell_idx_symbols for f in all_fields}
 
-    loop_strides = list(fields_without_buffers)[0].shape
-
     if any(FieldType.is_buffer(f) for f in all_fields):
-        resolve_buffer_accesses(ast, get_base_buffer_index(ast, indexing.coordinates, loop_strides), read_only_fields)
+        iteration_space = indexing.iteration_space(common_shape)
+        resolve_buffer_accesses(ast, get_base_buffer_index(ast, cell_idx_symbols, iteration_space), read_only_fields)
 
     resolve_field_accesses(ast, read_only_fields, field_to_base_pointer_info=base_pointer_info,
                            field_to_fixed_coordinates=coord_mapping)
@@ -104,14 +114,20 @@ def create_cuda_kernel(assignments,
     return ast
 
 
-def created_indexed_cuda_kernel(assignments,
-                                index_fields,
-                                function_name="kernel",
-                                type_info=None,
-                                coordinate_names=('x', 'y', 'z'),
-                                indexing_creator=BlockIndexing,
-                                use_textures_for_interpolation=True):
-    fields_read, fields_written, assignments = add_types(assignments, type_info, check_independence_condition=False)
+def created_indexed_cuda_kernel(assignments: Union[AssignmentCollection, NodeCollection],
+                                config: CreateKernelConfig):
+
+    index_fields = config.index_fields
+    function_name = config.function_name
+    coordinate_names = config.coordinate_names
+    indexing_creator = indexing_creator_from_params(config.gpu_indexing, config.gpu_indexing_params)
+
+    fields_written = assignments.bound_fields
+    fields_read = assignments.rhs_fields
+    assignments = assignments.all_assignments
+
+    assignments = add_types(assignments, config)
+
     all_fields = fields_read.union(fields_written)
     read_only_fields = set([f.name for f in fields_read - fields_written])
 
@@ -144,13 +160,11 @@ def created_indexed_cuda_kernel(assignments,
                                 iteration_slice=[slice(None, None, None)] * len(idx_field.spatial_shape))
 
     function_body = Block(coordinate_symbol_assignments + assignments)
-    function_body = indexing.guard(function_body, get_common_shape(index_fields))
-    ast = KernelFunction(function_body, 'gpu', 'gpucuda', make_python_function,
+    function_body = indexing.guard(function_body, get_common_field(index_fields).spatial_shape)
+    ast = KernelFunction(function_body, Target.GPU, Backend.CUDA, make_python_function,
                          None, function_name, assignments=assignments)
     ast.global_variables.update(indexing.index_variables)
 
-    implement_interpolations(ast, implement_by_texture_accesses=use_textures_for_interpolation)
-
     coord_mapping = indexing.coordinates
     base_pointer_spec = [['spatialInner0']]
     base_pointer_info = {f.name: parse_base_pointer_info(base_pointer_spec, [2, 1, 0],
